/**
 * eobjects.org MetaModel
 * Copyright (C) 2010 eobjects.org
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
package org.eobjects.metamodel.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import org.eobjects.metamodel.query.CompiledQuery;
import org.eobjects.metamodel.query.DefaultCompiledQuery;
import org.eobjects.metamodel.query.Query;

/**
 * Implementation of {@link CompiledQuery} for JDBC which uses a
 * {@link PreparedStatement} behind the scenes.
 */
final class JdbcCompiledQuery extends DefaultCompiledQuery implements CompiledQuery {

    private final JdbcDataContext _dataContext;
    private final Connection _con;
    private final String _sql;
    private final PreparedStatement _statement;
    private final Query _query;

    public JdbcCompiledQuery(JdbcDataContext dc, Query query) {
        super(query);
        _dataContext = dc;
        _con = dc.getConnection();
        _query = query;
        _sql = dc.getQueryRewriter().rewriteQuery(query);
        try {
            _statement = _con.prepareStatement(_sql);
        } catch (SQLException e) {
            _dataContext.close(_con, null, null);
            throw JdbcUtils.wrapException(e, "preparing statement");
        }
    }

    protected PreparedStatement getPreparedStatement() {
        return _statement;
    }

    protected Query getQuery() {
        return _query;
    }

    protected Connection getConnection() {
        return _con;
    }

    @Override
    public String toSql() {
        return _sql;
    }

    @Override
    public void close() {
        _dataContext.close(_con, null, _statement);
    }
}
